local *

{
  :LibStub
  :idTweaks
  :idTweaksUtils
  :idTweaksOptions
} = _G
{ :Option, :ToggleOption } = idTweaksOptions
{ :split, :join } = idTweaksUtils.string

AceConfig = LibStub 'AceConfig-3.0'
AceConfigDialog = LibStub 'AceConfigDialog-3.0'

class Module
  new: (addon, name, ...) =>
    -- print "Module#new:#{name}"

    @addon = addon
    @name = name

    @ace_module = @addon.ace_addon\NewModule name, ...

    @ace_module.OnInitialize = ->
      -- print "Module#new:#{name}:ace_module.OnInitialize"

      @set_database!
      @set_options!
      @set_defaults!
      @initialize!

    @ace_module.OnEnable = ->
      -- print "Module#new:#{name}:ace_module.OnEnable"

      @register_options!
      @check!

    @ace_module.OnDisable = ->
      -- print "Module#new:#{name}:ace_module.OnDisable"

      @disable!

  add_option: (option) =>
    -- print "Module#add_option:#{@name}:#{option.name}"

    @options[option.name] = option

  remove_option: (option) =>
    -- print "Module#remove_option:#{@name}:#{option.name}"

    @options[option.name] = nil

  set_database: =>
    -- print "Module#set_database:#{@name}"

    @db = @addon.db\RegisterNamespace @name, {}

  set_defaults: =>
    -- print "Module#set_defaults:#{@name}"

    defaults =
      profile: @options\get_aceconfig_defaults!

    @db\RegisterDefaults defaults

  set_options: =>
    -- print "Module#set_options:#{@name}"

    @options = Option @, 'profile', @name, @name, {},
      enabled: ToggleOption @, 'enabled', 'Enabled', 'Enables / Disables the Module'

  register_options: =>
    -- print "Module#register_options:#{@name}"

    AceConfig\RegisterOptionsTable @name, @options\get_aceconfig_options!
    AceConfigDialog\AddToBlizOptions @name, @name, @addon.name

  initialize: =>
    -- print "Module#initialize:#{@name}"

  check: =>
    -- print "Module#check:#{@name}"

    enabled_option = @options\get_option 'enabled'
    is_enabled = enabled_option\get_value!

    if is_enabled
      @enable!
      @positioning!
    else
      @disable!

  enable: =>
    -- print "Module#enable:#{@name}"

  positioning: =>
    -- print "Module#positioning:#{@name}"

  disable: =>
    -- print "Module#disable:#{@name}"

_G.idTweaksModule = Module
