local *

{
  :AcceptQuest
  :C_GossipInfo
  :CompleteQuest
  :GetNumActiveQuests
  :GetNumAvailableQuests
  :GetNumQuestChoices
  :IsAltKeyDown
  :IsControlKeyDown
  :IsQuestCompletable
  :IsShiftKeyDown
  :QuestFrameCompleteQuestButton
  :SelectActiveQuest
  :SelectAvailableQuest
  :ShowQuestComplete
  :UnitName
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :SelectOption
  :ToggleOption
} = idTweaksOptions

class Questing extends idTweaksModule
  new: (addon) =>
    -- print 'Questing#new'

    super addon, 'Questing', 'AceConsole-3.0', 'AceEvent-3.0'

  set_options: =>
    -- print 'Questing#set_options'

    super!

    @options\add_option SelectOption @, 'pause_key', 'Pause Key', 'While you hold this key nothing will happen', {
      values:
        shift: 'Shift'
        control: 'Control'
        alt: 'Alt'
    }, {}, 'shift'

    @options\add_option ToggleOption @, 'accept_quests', 'Accept Quests', 'Accept quests'
    @options\add_option ToggleOption @, 'complete_quests', 'Complete Quests', 'Automatically complete quests unless when offered a choice of rewards'
    @options\add_option ToggleOption @, 'complete_gossip', 'Complete Gossip', 'When the gossip window has one option, use it'
    @options\add_option ToggleOption @, 'complete_covenant_sanctum', 'Complete Seat of the Primus', 'Automate interactions in the Seat of the Primus'

  enable: =>
    -- print 'Questing#enable'

    @ace_module\RegisterEvent 'GOSSIP_SHOW', (...) -> @GOSSIP_SHOW ...
    @ace_module\RegisterEvent 'QUEST_AUTOCOMPLETE', (...) -> @QUEST_AUTOCOMPLETE ...
    @ace_module\RegisterEvent 'QUEST_COMPLETE', (...) -> @QUEST_COMPLETE ...
    @ace_module\RegisterEvent 'QUEST_DETAIL', (...) -> @QUEST_DETAIL ...
    @ace_module\RegisterEvent 'QUEST_GREETING', (...) -> @QUEST_GREETING ...
     -- @ace_module\RegisterEvent 'QUEST_LOG_UPDATE', (...) -> @QUEST_LOG_UPDATE ...
    @ace_module\RegisterEvent 'QUEST_PROGRESS', (...) -> @QUEST_PROGRESS ...

  disable: =>
    -- print 'Questing#disable'

    @ace_module\UnregisterEvent 'GOSSIP_SHOW'
    @ace_module\UnregisterEvent 'QUEST_AUTOCOMPLETE'
    @ace_module\UnregisterEvent 'QUEST_COMPLETE'
    @ace_module\UnregisterEvent 'QUEST_DETAIL'
    @ace_module\UnregisterEvent 'QUEST_GREETING'
    @ace_module\UnregisterEvent 'QUEST_LOG_UPDATE'
    @ace_module\UnregisterEvent 'QUEST_PROGRESS'

  should_automate: =>
    -- print 'Questing#should_automate'

    output = true
    key = @db.profile.pause_key

    if key == 'shift'
      if IsShiftKeyDown!
        output = false

    if key == 'control'
      if IsControlKeyDown!
        output = false

    if key == 'alt'
      if IsAltKeyDown!
        output = false

    output

  strip_text: (text) =>
    -- print 'Questing#strip_text'

    return unless text
    text = text\gsub '|c%x%x%x%x%x%x%x%x(.-)|r', '%1'
    text = text\gsub '%[.*%]%s*', ''
    text = text\gsub '(.+) %(.+%)', '%1'
    text = text\trim!
    text

  autocomplete_quest: (questId) =>
    -- print 'Questing#autocomplete_quest'

    return unless @db.profile.complete_quests

    ShowQuestComplete questId

  progress_quest: () =>
    -- print 'Questing#progress_quest'

    return unless @db.profile.complete_quests

    if IsQuestCompletable!
      CompleteQuest!

  complete_quest: () =>
    -- print 'Questing#complete_quest'

    return unless @db.profile.complete_quests

    if GetNumQuestChoices! <= 1
      QuestFrameCompleteQuestButton\Click!

  accept_quest: () =>
    -- print 'Questing#accept_quest'

    return unless @db.profile.accept_quests

    AcceptQuest!

  automate_gossip_window: () =>
    -- print 'Questing#automate_gossip_window'

     -- In the new API, complete quests and then accept them. You always get all progress in one click.
    num_active_quests = C_GossipInfo.GetNumActiveQuests!
     -- print "Questing#automate_gossip_window:num_active_quests #{num_active_quests}"
    if num_active_quests > 0
      for i = 1, num_active_quests
        C_GossipInfo.SelectActiveQuest i

    num_available_quests = C_GossipInfo.GetNumAvailableQuests!
     -- print "Questing#automate_gossip_window:num_available_quests #{num_available_quests}"
    if num_available_quests > 0
      for i = 1, num_available_quests
        C_GossipInfo.SelectAvailableQuest i

     -- In the old API, accept quests before completing quests. You might have to click multiple times.
    num_available_quests = GetNumAvailableQuests!
     -- print "Questing#automate_gossip_window:num_available_quests #{num_available_quests}"
    if num_available_quests > 0
      for i = 1, num_available_quests
        SelectAvailableQuest i

    num_active_quests = GetNumActiveQuests!
     -- print "Questing#automate_gossip_window:num_active_quests #{num_active_quests}"
    if num_active_quests > 0
      for i = 1, num_active_quests
        SelectActiveQuest i

    num_gossips = C_GossipInfo.GetNumOptions!
     -- print "Questing#automate_gossip_window:num_gossips #{num_gossips}"
    if num_gossips == 1
      C_GossipInfo.SelectOption 1

  QUEST_AUTOCOMPLETE: (event, questId) =>
    -- print 'Questing#QUEST_AUTOCOMPLETE', event, questId

    return unless @should_automate!

    @autocomplete_quest(questId)

  QUEST_COMPLETE: (event) =>
    -- print 'Questing#QUEST_COMPLETE'

    return unless @should_automate!

    @complete_quest!

  QUEST_PROGRESS: =>
    -- print 'Questing#QUEST_PROGRESS'

    return unless @should_automate!

    @progress_quest!

  GOSSIP_SHOW: =>
    -- print 'Questing#GOSSIP_SHOW'

    return unless @should_automate!

    if @db.profile.complete_covenant_sanctum and 'Arkadia Moa' == UnitName 'target'
      C_GossipInfo.SelectOption 1
      return

    @automate_gossip_window!

  QUEST_GREETING: (...) =>
    -- print 'Questing#QUEST_GREETING'

    return unless @should_automate!

    @automate_gossip_window!

  QUEST_DETAIL: =>
    -- print 'Questing#QUEST_DETAIL'

    return unless @should_automate!

    @accept_quest!

idTweaks\add_module Questing idTweaks
